import datetime
import logging
import time
import random
import requests
import sys
import yaml
import os
from flask import Flask, request, redirect, json, make_response

# 初始化日志配置
log = logging.getLogger('api_log')
log.setLevel(logging.INFO)
# 创建一个handler，用于写入日志文件
fh = logging.FileHandler('test.log')
fh.setLevel(logging.DEBUG)
# 再创建一个handler，用于输出到控制台
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
# 定义handler的输出格式
formatter = logging.Formatter('%(asctime)s-%(levelname)s: %(message)s')
fh.setFormatter(formatter)
ch.setFormatter(formatter)
# 给logger添加handler
log.addHandler(fh)
log.addHandler(ch)
run_path = '/Users/bytedance/Desktop/work_bytedance/PY_Scripts/third_src'


def readconf(m_type, m_name):
    if m_type == 'conf':
        m_path = '{}/config/{}.yaml'.format(run_path, m_name)
    elif m_type == 'cache':
        m_path = '{}/cache/{}.yaml'.format(run_path, m_name)
    with open(m_path, encoding='utf-8') as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
        key = data
    return key


def writeconf(m_type, m_name, data=None):
    if m_type == 'conf':
        m_path = '{}/config/{}.yaml'.format(run_path, m_name)
    elif m_type == 'cache':
        m_path = '{}/cache/{}.yaml'.format(run_path, m_name)
        # print(yaml.dump(data, allow_unicode=True, sort_keys=False))
    with open(m_path, 'w', encoding='utf-8') as f:
        f.write(yaml.dump(data, allow_unicode=True, sort_keys=False))


# 10000	非法的请求参数	Invalid param
# 10001	鉴权失败[key错误]	Authentication failed
# 10002	初始化失败，请检查配置信息是否正确  Initialization failed, please check the configuration information
# 10003 创建用户失败，失败原因：  Failed to {create/update/select/delete} user, failure reason:
# 10004
# 10005
# 10006
# 10007
# 10008
# 10009
def mxin_time():
    date01 = datetime.date.today()
    if date01.month < 10:
        month = '0{}'.format(date01.month)
    else:
        month = date01.month
    if date01.day < 10:
        day = '0{}'.format(date01.day)
    else:
        day = date01.day
    return '{}{}'.format(month, day)


app = Flask(__name__)
global bind_addr, bind_port, bind_debug
bind_addr = '127.0.0.1'
bind_port = 8003
bind_debug = False


@app.route('/', methods=['GET'])
def mxin_http():
    return 'Hello Word'


@app.route('/conf', methods=['POST'])
def mxin_config():
    try:
        # print(request.get_data())
        data = json.loads(request.get_data())
    except ValueError:
        status = {
            'code': 10000,
            'msg': 'Invalid param'
        }
        return status
    status = {
        'code': 0,
        'msg': 'success'
    }
    if not os.path.exists(run_path + '/config'):
        os.mkdir(run_path + '/config')
    writeconf('conf', 'conf', data)
    log.info('get config is {}'.format(data))
    return status


@app.route('/api/init', methods=['POST'])
def mxin_init():
    key = readconf('conf', 'key')['password']
    try:
        token = request.headers['Content-Key']
    except:
        status = {
            'code': 10001,
            'msg': 'Authentication failed'
        }
        return status
    if token != key:
        status = {
            'code': 10001,
            'msg': 'Authentication failed'
        }
        return status
    else:
        # 飞书获取token
        lark_url = 'https://open.feishu.cn/open-apis/auth/v3/tenant_access_token/internal'
        lark_headers = {
            'Content-Type': 'application/json; charset=utf-8'
        }
        lark_body = {
            'app_id': readconf('conf', 'conf')['lark']['app_id'],
            'app_secret': readconf('conf', 'conf')['lark']['app_secret']
        }
        lark_body = json.dumps(lark_body)
        lark_data = json.loads(requests.post(lark_url, data=lark_body, headers=lark_headers, ).text)
        if lark_data['code'] == 0:
            lark_status = {'code': 0, 'msg': 'success'}
            lark_token = {'token': lark_data['tenant_access_token']}
            writeconf('cache', 'lark_token', lark_token)
            log.info('lark accesstoken is {}'.format(lark_data['tenant_access_token']))
        else:
            lark_status = {'code': lark_data['code'], 'msg': lark_data['msg']}

        # authing获取token
        authing_url = 'https://api.authing.cn/api/v3/get-management-token'
        authing_headers = {
            'Content-Type': 'application/json'
        }
        authing_body = {
            'accessKeyId': readconf('conf', 'conf')['authing']['app_id'],
            'accessKeySecret': readconf('conf', 'conf')['authing']['app_secret']
        }
        authing_body = json.dumps(authing_body)
        authing_data = json.loads(requests.post(authing_url, data=authing_body, headers=authing_headers).text)
        if authing_data['statusCode'] == 200:
            authing_status = {'code': 0, 'msg': 'success'}
            authing_token = {'token': authing_data['data']['access_token']}
            writeconf('cache', 'authing_token', authing_token)
            log.info('authing accesstoken is {}'.format(authing_data['data']['access_token']))
        else:
            authing_status = {'code': authing_data['statusCode'], 'msg': authing_data['message']}

        # dingtalk获取token
        dingtalk_url = 'https://oapi.dingtalk.com/gettoken?appkey={}&appsecret={}'.format(
            readconf('conf', 'conf')['dingtalk']['app_id'], readconf('conf', 'conf')['dingtalk']['app_secret'])
        dingtalk_data = json.loads(requests.get(dingtalk_url).text)
        if dingtalk_data['errcode'] == 0:
            dingtalk_status = {'code': 0, 'msg': 'success'}
            dingtalk_token = {'token': dingtalk_data['access_token']}
            writeconf('cache', 'dingtalk_token', dingtalk_token)
            log.info('dingtalk accesstoken is {}'.format(dingtalk_data['access_token']))
        else:
            dingtalk_status = {'code': dingtalk_data['errcode'], 'msg': dingtalk_data['errmsg']}

        # weixin获取token
        weixin_url = 'https://qyapi.weixin.qq.com/cgi-bin/gettoken?corpid={}&corpsecret={}'.format(readconf('conf', 'conf')['weixin']['app_id'], readconf('conf', 'conf')['weixin']['app_secret'])
        weixin_data = json.loads(requests.get(weixin_url).text)
        if weixin_data['errcode'] == 0:
            weixin_status = {'code': 0, 'msg': 'success'}
            weixin_token = {'token': weixin_data['access_token']}
            writeconf('cache', 'weixin_token', weixin_token)
            log.info('weixin accesstoken is {}'.format(weixin_data['access_token']))
        else:
            weixin_status = {'code': weixin_data['errcode'], 'msg': weixin_data['errmsg']}

        # 最终返回
        if lark_data['code'] != 0 or authing_data['statusCode'] != 200 or dingtalk_data['errcode'] != 0 or weixin_data['errcode'] != 0:
            status = {'code': 10002, 'msg': 'Initialization failed, please check the configuration information',
                      'data': {'lark': lark_status, 'authing': authing_status, 'dingtalk': dingtalk_status, 'weixin': weixin_status,
                               'saved_config': readconf('conf', 'conf')}}
            return status
        else:
            status = {'code': 0, 'msg': 'init success'}
            return status


@app.route('/api/cmd', methods=['POST'])
def mxin_cmd():
    try:
        data = json.loads(request.get_data())
    except ValueError:
        status = {
            'code': 10000,
            'msg': 'Invalid param'
        }
        return status
    cmd_src = data['source']
    cmd_cmd = data['command']
    cmd_type = data['type']  # 为部门和人员类型区分预览
    if cmd_src == 'lark':
        dpt_open_id_lark = readconf('conf', 'conf')['lark']['dpt_open_id_lark']
        if cmd_type != '' and cmd_type == 'usr':
            header = {'Authorization': 'Bearer ' + readconf('cache', 'lark_token')['token'],
                      'Content-Type': 'application/json; charset=utf-8'}
            if cmd_cmd != '':
                url = 'https://open.feishu.cn/open-apis/contact/v3/users/find_by_department?department_id_type=open_department_id&department_id={}'.format(
                    dpt_open_id_lark)
                resdata_list = json.loads(requests.get(url, headers=header).text)
            number = '{}{}'.format(random.randint(1000, 9999), mxin_time())
            if cmd_cmd != '' and cmd_cmd == 'create':
                url = 'https://open.feishu.cn/open-apis/contact/v3/users'
                body = {
                    'name': 'apitest-入-{}-lark'.format(number),
                    'mobile': '136{}'.format(number),
                    'email': 'apitest-{}@mxin.me'.format(number),
                    'department_ids': [dpt_open_id_lark],
                    'employee_type': 1,

                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('lark add user info is: {}'.format(resdata))
                if resdata['code'] == 0 and resdata['msg'] == 'success':
                    status = {'code': 0, 'msg': 'add user success', 'name': '{}'.format(resdata['data']['user']['name'])}
                else:
                    status = {'code': 10003, 'msg': 'Failed to create user, failure reason: {}'.format(resdata['msg'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'update':
                try:
                    url = 'https://open.feishu.cn/open-apis/contact/v3/users/{}?user_id_type=open_id'.format(
                        resdata_list['data']['items'][1]['open_id'])
                except KeyError as e:
                    log.info(e)
                body = json.dumps({'name': 'apitest-改-{}'.format(mxin_time())})
                resdata = json.loads(requests.patch(url, headers=header, data=body).text)
                log.info('lark update user info is: {}'.format(resdata))
                if resdata['code'] == 0 and resdata['msg'] == 'success':
                    status = {'code': 0, 'msg': 'update user success', 'name': '{}'.format(resdata['data']['user']['name'])}
                else:
                    status = {'code': 10003, 'msg': 'Failed to update user, failure reason: {}'.format(resdata['msg'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'delete':
                try:
                    url = 'https://open.feishu.cn/open-apis/contact/v3/users/{}?user_id_type=open_id'.format(
                        resdata_list['data']['items'][0]['open_id'])
                except KeyError as e:
                    log.info(e)
                resdata = json.loads(requests.delete(url, headers=header).text)
                log.info('lark delete user info is: {}'.format(resdata))
                if resdata['code'] == 0 and resdata['msg'] == 'success':
                    status = {'code': 0, 'msg': 'delete user success'}
                else:
                    status = {'code': 10003, 'msg': 'Failed to delete user, failure reason: {}'.format(resdata['msg'])}
                return status
            else:
                status = {'code': 10000, 'msg': 'Invalid param'}
                return status
        elif cmd_type != '' and cmd_type == 'dpt':
            return 'dpt debug'
    elif cmd_src == 'authing':
        dpt_open_id_authing = readconf('conf', 'conf')['authing']['dpt_open_id_authing']
        if cmd_type != '' and cmd_type == 'usr':
            header = {'Authorization': readconf('cache', 'authing_token')['token'], 'Content-Type': 'application/json',
                      'x-authing-userpool-id': readconf('conf', 'conf')['authing']['user_pool_id']}
            if cmd_cmd != '':
                url = 'https://api.authing.cn/api/v3/list-users'
                body = {
                    'advancedFilter': [
                        {
                            'field': 'department',
                            'operator': 'IN',
                            'value': [
                                {
                                    'organizationCode': '{}'.format(
                                        readconf('conf', 'conf')['authing']['organizationCode']),
                                    'departmentId': '{}'.format(dpt_open_id_authing),
                                    'departmentIdType': 'department_id',
                                    'includeChildrenDepartments': True
                                }
                            ]
                        }
                    ]
                }
                body = json.dumps(body)
                resdata_list = json.loads(requests.post(url, headers=header, data=body).text)
            numbers = '{}{}'.format(random.randint(1000, 9999), mxin_time())
            if cmd_cmd != '' and cmd_cmd == 'create':
                url = 'https://api.authing.cn/api/v3/create-user'
                body = {
                    'username': 'apitest-入-{}-authing'.format(numbers),
                    'departmentIds': [dpt_open_id_authing],
                    'email': 'apitest-134{}@mxin.me'.format(numbers),
                    'phone': '134{}'.format(numbers),
                    'externalId': '{}{}'.format(mxin_time(), random.randint(10000, 99999))
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('authing add user info is: {}'.format(resdata))
                if resdata['statusCode'] == 200 and resdata['message'] == '':
                    status = {'code': 0, 'msg': 'add user success', 'name': '{}'.format(resdata['data']['username'])}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to create user, failure reason: {}'.format(resdata['message'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'update':
                url = 'https://api.authing.cn/api/v3/update-user'
                body = {
                    'username': 'apitest-改-{}-authing'.format(mxin_time()),
                    'departmentIds': [dpt_open_id_authing],
                    'email': 'apitest-134{}@mxin.me'.format(numbers),
                    'phone': '134{}'.format(numbers),
                    'userId': '{}'.format(resdata_list['data']['list'][1]['userId'])
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('authing update user info is: {}'.format(resdata))
                if resdata['statusCode'] == 200 and resdata['message'] == '':
                    status = {'code': 0, 'msg': 'update user success', 'name': '{}'.format(resdata['data']['username'])}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to update user, failure reason: {}'.format(resdata['message'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'delete':
                print(resdata_list['data']['list'][0]['userId'])
                url = 'https://api.authing.cn/api/v3/resign-user'
                body = {
                    'userId': "{}".format(resdata_list['data']['list'][0]['userId'])
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('authing delete user info is: {}'.format(resdata))
                if resdata['statusCode'] == 200 and resdata['message'] == '':
                    status = {'code': 0, 'msg': 'delete user success'}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to delete user, failure reason: {}'.format(resdata['message'])}
                return status
    elif cmd_src == 'dingtalk':
        dpt_open_id_dingtalk = readconf('conf', 'conf')['dingtalk']['dpt_open_id_dingtalk']
        token = readconf('cache', 'dingtalk_token')['token']
        if cmd_type != '' and cmd_type == 'usr':
            header = {}
            if cmd_cmd != '':
                url = 'https://oapi.dingtalk.com/topapi/user/listid?access_token={}'.format(token)
                body = {
                    'dept_id': dpt_open_id_dingtalk
                }
                body = json.dumps(body)
                resdata_list = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('resdate info is: {}'.format(resdata_list))
            numbers = '{}{}'.format(random.randint(1000, 9999), mxin_time())
            if cmd_cmd != '' and cmd_cmd == 'create':
                url = 'https://oapi.dingtalk.com/topapi/v2/user/create?access_token={}'.format(token)
                body = {
                    'name': 'apitest-入-{}-dingtalk'.format(numbers),
                    'mobile': '135{}'.format(numbers),
                    'email': 'apitest-135{}@mxin.me'.format(numbers),
                    'dept_id_list': '{}'.format(dpt_open_id_dingtalk)
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('dingtalk add user info is: {}'.format(resdata))
                if resdata['errcode'] == 0 and resdata['errmsg'] == 'ok':
                    status = {'code': 0, 'msg': 'add user success'}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to create user, failure reason: {}'.format(resdata['errmsg'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'update':
                url = 'https://oapi.dingtalk.com/topapi/v2/user/update?access_token={}'.format(token)
                body = {
                    'userid': '{}'.format(resdata_list['result']['userid_list'][1]),
                    'name': 'apitest-改-{}-dingtalk'.format(mxin_time()),
                    'email': 'apitest-134{}@mxin.me'.format(numbers)
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('dingtalk update user info is: {}'.format(resdata))
                if resdata['errcode'] == 0 and resdata['errmsg'] == 'ok':
                    status = {'code': 0, 'msg': 'update user success'}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to update user, failure reason: {}'.format(resdata['message'])}
                return status
            elif cmd_cmd != '' and cmd_cmd == 'delete':
                url = 'https://oapi.dingtalk.com/topapi/v2/user/delete?access_token={}'.format(token)
                body = {
                    'userid': '{}'.format(resdata_list['result']['userid_list'][0])
                }
                body = json.dumps(body)
                resdata = json.loads(requests.post(url, headers=header, data=body).text)
                log.info('dingtalk delete user info is: {}'.format(resdata))
                if resdata['errcode'] == 0 and resdata['errmsg'] == 'ok':
                    status = {'code': 0, 'msg': 'delete user success'}
                else:
                    status = {'code': 10003,
                              'msg': 'Failed to update user, failure reason: {}'.format(resdata['message'])}
                return status
    elif cmd_src == 'weixin':
        dpt_open_id_weixin = readconf('conf', 'conf')['weixin']['dpt_open_id_weixin']
        header = {}
        token = readconf('cache', 'weixin_token')['token']
        if cmd_cmd != '':
            url = 'https://oapi.dingtalk.com/topapi/user/listid?access_token={}'.format(token)
            body = {
                'dept_id': dpt_open_id_weixin
            }
            body = json.dumps(body)
            resdata_list = json.loads(requests.post(url, headers=header, data=body).text)
            log.info('resdate info is: {}'.format(resdata_list))
        numbers = '{}{}'.format(random.randint(1000, 9999), mxin_time())
        if cmd_cmd != '' and cmd_cmd == 'create':
            url = 'https://qyapi.weixin.qq.com/cgi-bin/user/create?access_token={}'.format(token)
            body = {
                'userid': '{}'.format(round(time.time() * 1000)),
                'name': 'apitest-入-{}-weixin'.format(numbers),
                'mobile': '131{}'.format(numbers),
                'email': 'apitest-131{}@mxin.me'.format(numbers),
                'department': '{}'.format(dpt_open_id_weixin)
            }
            body = json.dumps(body)
            resdata = json.loads(requests.post(url, headers=header, data=body).text)
            log.info('weixin add user info is: {}'.format(resdata))
            if resdata['errcode'] == 0 and resdata['errmsg'] == 'ok':
                status = {'code': 0, 'msg': 'add user success'}
            else:
                status = {'code': 10003,
                          'msg': 'Failed to create user, failure reason: {}'.format(resdata['errmsg'])}
            return status
        elif cmd_cmd != '' and cmd_cmd == 'update':
            url = 'https://qyapi.weixin.qq.com/cgi-bin/user/update?access_token={}'.format(token)
            body = {
                'userid': '{}'.format(resdata_list['result']['userid_list'][1]),
                'name': 'apitest-改-{}-weixin'.format(mxin_time()),
                'email': 'apitest-131{}@mxin.me'.format(numbers)
            }
            body = json.dumps(body)
            resdata = json.loads(requests.post(url, headers=header, data=body).text)
            log.info('weixin update user info is: {}'.format(resdata))
            if resdata['errcode'] == 0 and resdata['errmsg'] == 'ok':
                status = {'code': 0, 'msg': 'update user success'}
            else:
                status = {'code': 10003,
                          'msg': 'Failed to update user, failure reason: {}'.format(resdata['message'])}
            return status
        elif cmd_cmd != '' and cmd_cmd == 'delete':
            return True
    else:
        status = {'code': 10000, 'msg': 'Invalid param'}
        return status


app.run(bind_addr, bind_port, bind_debug)
